@@ -1,12 +1,26 @@
-# -*- coding: utf-8 -*-
 # emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
 # vi: set ft=python sts=4 ts=4 sw=4 et:
-"""
-Helpers for handling BIDS-like neuroimaging structures
-^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
-
-
-"""
+#
+# Copyright 2021 The NiPreps Developers <nipreps@gmail.com>
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+#
+# We support and encourage derived works from this project, please read
+# about our expectations at
+#
+#     https://www.nipreps.org/community/licensing/
+#
+"""Helpers for handling BIDS-like neuroimaging structures."""
 from pathlib import Path
 import json
 import re
@@ -19,10 +33,13 @@ class BIDSError(ValueError):
     def __init__(self, message, bids_root):
         indent = 10
         header = '{sep} BIDS root folder: "{bids_root}" {sep}'.format(
-            bids_root=bids_root, sep=''.join(['-'] * indent))
-        self.msg = '\n{header}\n{indent}{message}\n{footer}'.format(
-            header=header, indent=''.join([' '] * (indent + 1)),
-            message=message, footer=''.join(['-'] * len(header))
+            bids_root=bids_root, sep="".join(["-"] * indent)
+        )
+        self.msg = "\n{header}\n{indent}{message}\n{footer}".format(
+            header=header,
+            indent="".join([" "] * (indent + 1)),
+            message=message,
+            footer="".join(["-"] * len(header)),
         )
         super(BIDSError, self).__init__(self.msg)
         self.bids_root = bids_root
@@ -32,27 +49,38 @@ class BIDSWarning(RuntimeWarning):
     pass
 
 
-def collect_participants(bids_dir, participant_label=None, strict=False,
-                         bids_validate=True):
+def collect_participants(
+    bids_dir, participant_label=None, strict=False, bids_validate=True
+):
     """
     List the participants under the BIDS root and checks that participants
     designated with the participant_label argument exist in that folder.
     Returns the list of participants to be finally processed.
     Requesting all subjects in a BIDS directory root:
+
+    .. testsetup::
+
+        >>> data_dir_canary()
+
+    Examples
+    --------
     >>> collect_participants(str(datadir / 'ds114'), bids_validate=False)
     ['01', '02', '03', '04', '05', '06', '07', '08', '09', '10']
 
     Requesting two subjects, given their IDs:
+
     >>> collect_participants(str(datadir / 'ds114'), participant_label=['02', '04'],
     ...                      bids_validate=False)
     ['02', '04']
 
     Requesting two subjects, given their IDs (works with 'sub-' prefixes):
+
     >>> collect_participants(str(datadir / 'ds114'), participant_label=['sub-02', 'sub-04'],
     ...                      bids_validate=False)
     ['02', '04']
 
     Requesting two subjects, but one does not exist:
+
     >>> collect_participants(str(datadir / 'ds114'), participant_label=['02', '14'],
     ...                      bids_validate=False)
     ['02']
@@ -62,6 +90,7 @@ def collect_participants(bids_dir, participant_label=None, strict=False,
     Traceback (most recent call last):
     BIDSError:
     ...
+
     """
 
     if isinstance(bids_dir, BIDSLayout):
@@ -74,11 +103,13 @@ def collect_participants(bids_dir, participant_label=None, strict=False,
     # Error: bids_dir does not contain subjects
     if not all_participants:
         raise BIDSError(
-            'Could not find participants. Please make sure the BIDS data '
-            'structure is present and correct. Datasets can be validated online '
-            'using the BIDS Validator (http://bids-standard.github.io/bids-validator/).\n'
-            'If you are using Docker for Mac or Docker for Windows, you '
-            'may need to adjust your "File sharing" preferences.', bids_dir)
+            "Could not find participants. Please make sure the BIDS data "
+            "structure is present and correct. Datasets can be validated online "
+            "using the BIDS Validator (http://bids-standard.github.io/bids-validator/).\n"
+            "If you are using Docker for Mac or Docker for Windows, you "
+            'may need to adjust your "File sharing" preferences.',
+            bids_dir,
+        )
 
     # No --participant-label was set, return all
     if not participant_label:
@@ -88,20 +119,26 @@ def collect_participants(bids_dir, participant_label=None, strict=False,
         participant_label = [participant_label]
 
     # Drop sub- prefixes
-    participant_label = [sub[4:] if sub.startswith('sub-') else sub for sub in participant_label]
+    participant_label = [
+        sub[4:] if sub.startswith("sub-") else sub for sub in participant_label
+    ]
     # Remove duplicates
     participant_label = sorted(set(participant_label))
     # Remove labels not found
     found_label = sorted(set(participant_label) & all_participants)
     if not found_label:
-        raise BIDSError('Could not find participants [{}]'.format(
-            ', '.join(participant_label)), bids_dir)
+        raise BIDSError(
+            "Could not find participants [{}]".format(", ".join(participant_label)),
+            bids_dir,
+        )
 
     # Warn if some IDs were not found
     notfound_label = sorted(set(participant_label) - all_participants)
     if notfound_label:
-        exc = BIDSError('Some participants were not found: {}'.format(
-            ', '.join(notfound_label)), bids_dir)
+        exc = BIDSError(
+            "Some participants were not found: {}".format(", ".join(notfound_label)),
+            bids_dir,
+        )
         if strict:
             raise exc
         warnings.warn(exc.msg, BIDSWarning)
@@ -109,10 +146,23 @@ def collect_participants(bids_dir, participant_label=None, strict=False,
     return found_label
 
 
-def collect_data(bids_dir, participant_label, task=None, echo=None,
-                 bids_validate=True, bids_filters=None):
+def collect_data(
+    bids_dir,
+    participant_label,
+    task=None,
+    echo=None,
+    bids_validate=True,
+    bids_filters=None,
+):
     """
     Uses pybids to retrieve the input data for a given participant
+
+    .. testsetup::
+
+        >>> data_dir_canary()
+
+    Examples
+    --------
     >>> bids_root, _ = collect_data(str(datadir / 'ds054'), '100185',
     ...                             bids_validate=False)
     >>> bids_root['fmap']  # doctest: +ELLIPSIS
@@ -141,6 +191,7 @@ def collect_data(bids_dir, participant_label, task=None, echo=None,
     ...                             bids_validate=False, bids_filters={'t1w':{'run': 1}})
     >>> bids_root['t1w']  # doctest: +ELLIPSIS
     ['.../ds051/sub-01/anat/sub-01_run-01_T1w.nii.gz']
+
     """
     if isinstance(bids_dir, BIDSLayout):
         layout = bids_dir
@@ -148,50 +199,61 @@ def collect_data(bids_dir, participant_label, task=None, echo=None,
         layout = BIDSLayout(str(bids_dir), validate=bids_validate)
 
     queries = {
-        'fmap': {'datatype': 'fmap'},
-        'bold': {'datatype': 'func', 'suffix': 'bold'},
-        'sbref': {'datatype': 'func', 'suffix': 'sbref'},
-        'flair': {'datatype': 'anat', 'suffix': 'FLAIR'},
-        't2w': {'datatype': 'anat', 'suffix': 'T2w'},
-        't1w': {'datatype': 'anat', 'suffix': 'T1w'},
-        'roi': {'datatype': 'anat', 'suffix': 'roi'},
+        "fmap": {"datatype": "fmap"},
+        "bold": {"datatype": "func", "suffix": "bold", "part": ["mag", None]},
+        "sbref": {"datatype": "func", "suffix": "sbref", "part": ["mag", None]},
+        "flair": {"datatype": "anat", "suffix": "FLAIR", "part": ["mag", None]},
+        "t2w": {"datatype": "anat", "suffix": "T2w", "part": ["mag", None]},
+        "t1w": {"datatype": "anat", "suffix": "T1w", "part": ["mag", None]},
+        "roi": {"datatype": "anat", "suffix": "roi"},
     }
     bids_filters = bids_filters or {}
     for acq, entities in bids_filters.items():
         queries[acq].update(entities)
 
     if task:
-        queries['bold']['task'] = task
+        queries["bold"]["task"] = task
 
     if echo:
-        queries['bold']['echo'] = echo
+        queries["bold"]["echo"] = echo
 
     subj_data = {
-        dtype: sorted(layout.get(return_type='file', subject=participant_label,
-                                 extension=['nii', 'nii.gz'], **query))
-        for dtype, query in queries.items()}
+        dtype: sorted(
+            layout.get(
+                return_type="file",
+                subject=participant_label,
+                extension=["nii", "nii.gz"],
+                **query,
+            )
+        )
+        for dtype, query in queries.items()
+    }
 
     # Special case: multi-echo BOLD, grouping echos
-    if any(['_echo-' in bold for bold in subj_data['bold']]):
-        subj_data['bold'] = group_multiecho(subj_data['bold'])
+    if any(["_echo-" in bold for bold in subj_data["bold"]]):
+        subj_data["bold"] = group_multiecho(subj_data["bold"])
 
     return subj_data, layout
 
 
 def get_metadata_for_nifti(in_file, bids_dir=None, validate=True):
-    """Fetch metadata for a given nifti file
+    """
+    Fetch metadata for a given NIfTI file.
 
+    .. testsetup::
+
+        >>> data_dir_canary()
+
+    Examples
+    --------
     >>> metadata = get_metadata_for_nifti(
     ...     datadir / 'ds054' / 'sub-100185' / 'fmap' / 'sub-100185_phasediff.nii.gz',
     ...     validate=False)
     >>> metadata['Manufacturer']
     'SIEMENS'
 
-    >>>
-
     """
-    return _init_layout(in_file, bids_dir, validate).get_metadata(
-        str(in_file))
+    return _init_layout(in_file, bids_dir, validate).get_metadata(str(in_file))
 
 
 def _init_layout(in_file=None, bids_dir=None, validate=True):
@@ -201,12 +263,12 @@ def _init_layout(in_file=None, bids_dir=None, validate=True):
     if bids_dir is None:
         in_file = Path(in_file)
         for parent in in_file.parents:
-            if parent.name.startswith('sub-'):
+            if parent.name.startswith("sub-"):
                 bids_dir = parent.parent.resolve()
                 break
 
         if bids_dir is None:
-            raise RuntimeError('Could not infer BIDS root')
+            raise RuntimeError("Could not infer BIDS root")
 
     layout = BIDSLayout(str(bids_dir), validate=validate)
     return layout
@@ -214,9 +276,12 @@ def _init_layout(in_file=None, bids_dir=None, validate=True):
 
 def group_multiecho(bold_sess):
     """
-    Multiplexes multi-echo EPIs into arrays. Dual-echo is a special
-    case of multi-echo, which is treated as single-echo data.
+    Multiplex multi-echo EPIs into arrays.
 
+    Dual-echo is a special case of multi-echo, which is treated as single-echo data.
+
+    Examples
+    --------
     >>> bold_sess = ["sub-01_task-rest_echo-1_run-01_bold.nii.gz",
     ...              "sub-01_task-rest_echo-2_run-01_bold.nii.gz",
     ...              "sub-01_task-rest_echo-1_run-02_bold.nii.gz",
@@ -262,8 +327,7 @@ def group_multiecho(bold_sess):
       'sub-01_task-beh_echo-3_run-02_bold.nii.gz'],
       'sub-01_task-beh_run-03_bold.nii.gz']
 
-    Some tests from https://neurostars.org/t/fmriprep-from\
--singularity-unboundlocalerror/3299/7
+    Some tests from https://neurostars.org/t/fmriprep-from-singularity-unboundlocalerror/3299/7
 
     >>> bold_sess = ['sub-01_task-AudLoc_echo-1_bold.nii',
     ...              'sub-01_task-AudLoc_echo-2_bold.nii',
@@ -285,12 +349,11 @@ def group_multiecho(bold_sess):
     >>> len(groups[-1])
     3
 
-
     """
     from itertools import groupby
 
     def _grp_echos(x):
-        if '_echo-' not in x:
+        if "_echo-" not in x:
             return x
         echo = re.search("_echo-\\d*", x).group(0)
         return x.replace(echo, "_echo-?")
@@ -299,7 +362,7 @@ def group_multiecho(bold_sess):
     for _, bold in groupby(bold_sess, key=_grp_echos):
         bold = list(bold)
         # If single- or dual-echo, flatten list; keep list otherwise.
-        action = getattr(ses_uids, 'append' if len(bold) > 2 else 'extend')
+        action = getattr(ses_uids, "append" if len(bold) > 2 else "extend")
         action(bold)
     return ses_uids
 
@@ -308,8 +371,8 @@ def relative_to_root(path):
     """
     Calculate the BIDS root folder given one file path's.
 
-    Example
-    -------
+    Examples
+    --------
     >>> str(relative_to_root(
     ...     "/sub-03/sourcedata/sub-01/anat/sub-01_T1.nii.gz"
     ... ))
@@ -346,7 +409,7 @@ def relative_to_root(path):
 
 def check_pipeline_version(cvers, data_desc):
     """
-    Searches for existing BIDS pipeline output and compares against current pipeline version.
+    Search for existing BIDS pipeline output and compares against current pipeline version.
 
     .. testsetup::
 
@@ -359,8 +422,8 @@ def check_pipeline_version(cvers, data_desc):
     ----------
     cvers : :obj:`str`
         Current pipeline version
-    data_desc : :obj:`str` or :class:`os.Pathlike`
-        Path to pipeline output's dataset_description.json
+    data_desc : :obj:`str` or :obj:`os.PathLike`
+        Path to pipeline output's ``dataset_description.json``
 
     Examples
     --------
